# -*- coding: utf-8 -*-
# *** FORMAT PARAMS ***


UPDATE_DYNAMIC_VARIABLES = [
    'args',
    'display_height',
    'display_width',
    'finished',
    'focused',
    'frame_count',
    'height',
    'is_dead',
    'is_dead_from_error',
    'is_ready',
    'is_running',
    'java_platform',
    'java_version',
    'java_version_name',
    'key',
    'key_code',
    'mouse_button',
    'mouse_x',
    'mouse_y',
    'np_pixels',
    'pixel_height',
    'pixel_width',
    'pixels',
    'pmouse_x',
    'pmouse_y',
    'width']
